#include <memory>
#include <vector>
#include <iostream>

class Gene{
     int32_t id;
     std::string code;
public:
     Gene(): id(-1), code(""){}
     Gene(uint32_t _id, std::string _code): id(_id), code(_code){}
     void print_gene(){
          std::cout << "Gene id: " << id
          << ", code: " << code << std::endl;
     }
};

int main(){

     // memory leak code:
     std::vector<Gene*> gene_vec;
     Gene* gene1 = new Gene(0, "ITIIIIITYYYYATTAAAIYTAIITAATIAIYAITYTAYYAIIATAYYIYITIAIYIIITTTAYTTAITYYYAY");
     gene_vec.emplace_back(gene1);


     // attempt0. doesn't work, using in place constructor. 
     // std::vector<Gene*> gene_vec;
     // gene_vec.emplace_back(new Gene(0, "ITIIIIITYYYYATTAAAIYTAIITAATIAIYAITYTAYYAIIATAYYIYITIAIYIIITTTAYTTAITYYYAY"));
     
     // attempt1. doesn't work
     // delete gene_vec;  // this doesn't work because delete only works for pointers.

     /** fix0 & 1. Use delete for heap-based objects. no memory leak, but not suggested. 
      *  Need to remember to also not use gene_vec later.
      *  Otherwise if try to access gene_vec after freeing memory, e.g. gene_vec[0]->print_gene();
      *  it would be undefined behaviour
      */
     // fix0: 
     // delete gene1;
     // fix1:
     // for(Gene* gene_ptr: gene_vec){
     //      delete gene_ptr;
     // }
     // gene_vec.clear();

     // fix1: use stack instead of heap
     

     // fix2: use smart pointers(unique_ptr or shared_ptr)
     std::vector<std::unique_ptr<Gene>> gene_vec;
     gene_vec.emplace_back(std::make_unique<Gene>(0, "ITIIIIITYYYYATTAAAIYTAIITAATIAIYAITYTAYYAIIATAYYIYITIAIYIIITTTAYTTAITYYYAY"));


     return 0;
}